/**
 * @file        ADSO-50691
 * @description Update minimum duration that shift pattern must be utilized
 * @author      Clarence (ccn7@3ds.com)
 * @copyright   Dassault Systèmes
 */
import { AppMP, Demo, Scenario } from '../../libmp/appmp';
import { qCustomMatcher } from '../../e2elib/lib/src/main/qmatchers.const';
import { StepDialog } from '../../libappsop/dialogsop';
import { DurationInputHelper } from '../../libappsop/durationselectorsop';
import { StepMatrix } from '../../libappsop/matrixeditorsop';
import { DialogShiftPattern, DialogShiftPatternFields } from '../../libmp/dialogs/dialog.shiftpattern';
import { MatrixEditorShiftPattern, MatrixEditorShiftPatternContextMenuItem } from '../../libmp/forms/form.shiftpattern';

describe('ADSO-50691 - Update minimum duration that shift pattern must be utilized', () => {
  const appMP = AppMP.getInstance();
  const meShiftPattern = appMP.viewCapacitiesTime.formShiftPattern.meShiftPattern;
  let dlgShiftPattern: DialogShiftPattern;
  // Test result
  const verifyCreateDefaultDurationZero: DialogShiftPatternFields = {MinimumDuration: DurationInputHelper.getDurationString({Days: 0, Hours: 0, Minutes: 0})};
  const updateMinDuration40Days: DialogShiftPatternFields = {MinimumDuration: DurationInputHelper.getDurationString({Days: 40})};
  const verifyInvalidDurationNegative: DialogShiftPatternFields = {MinimumDuration: DurationInputHelper.getDurationString({Days: -1})};
  const verifyValidDuration1Day: DialogShiftPatternFields = {MinimumDuration: DurationInputHelper.getDurationString({Days: 1})};
  const verifyValidDuration0Day: DialogShiftPatternFields = {MinimumDuration: DurationInputHelper.getDurationString({Days: 0})};
  const verifyValidDuration40Days: DialogShiftPatternFields = updateMinDuration40Days;

  beforeAll(async () => {
    jasmine.addMatchers(qCustomMatcher);
    await appMP.login();
  });

  afterAll(async () => {
    await appMP.resetActiveView(appMP.viewCapacitiesTime);
    await appMP.cleanupAndLogout();
  });

  afterEach(async () => {
    await appMP.checkToastMessage();
  });

  it(`Setup - ${AppMP.getDemoDataPath(Demo.Food, Scenario.Base)}`, async () => {
    await appMP.createDemoDataset(Demo.Food, Scenario.Base, false);
  });

  it(`Step 1 - Open view ${appMP.viewCapacitiesTime.viewPath}.`, async () => {
    await appMP.viewCapacitiesTime.switchTo();
  });

  it(`Step 2 - ${StepMatrix.rightClickMatrixSelectContextmenu(MatrixEditorShiftPattern.title, MatrixEditorShiftPatternContextMenuItem.Create.Label)}`, async () => {
    dlgShiftPattern = await meShiftPattern.rightClickMatrixSelectContextmenu(MatrixEditorShiftPatternContextMenuItem.Create.Name);
  });

  it(`Step 3 - ${StepDialog.verifyDialogValues(DialogShiftPattern.title, verifyCreateDefaultDurationZero)} ${StepDialog.clickCancel()}`, async () => {
    await dlgShiftPattern.verifyDialogValues(verifyCreateDefaultDurationZero);
    await dlgShiftPattern.clickCancel();
  });

  it(`Step 4 - ${StepMatrix.rightClickRowSelectContextmenu(MatrixEditorShiftPattern.title, '24x5', MatrixEditorShiftPatternContextMenuItem.Edit.Label)}`, async () => {
    dlgShiftPattern = await meShiftPattern.rightClickRowSelectContextmenu(MatrixEditorShiftPatternContextMenuItem.Edit.Name, '24x5');
  });

  it(`Step 5 - ${StepDialog.updateDialogValues(DialogShiftPattern.title, updateMinDuration40Days)} ${StepDialog.clickOK()}`, async () => {
    await dlgShiftPattern.updateDialogValues(updateMinDuration40Days);
    await dlgShiftPattern.clickOK();
  });

  it(`Step 6 - ${StepMatrix.rightClickRowSelectContextmenu(MatrixEditorShiftPattern.title, '24x5', MatrixEditorShiftPatternContextMenuItem.Edit.Label)}`, async () => {
    dlgShiftPattern = await meShiftPattern.rightClickRowSelectContextmenu(MatrixEditorShiftPatternContextMenuItem.Edit.Name, '24x5');
  });

  it(`Step 7 - ${StepDialog.verifyDialogValues(DialogShiftPattern.title, updateMinDuration40Days)}`, async () => {
    await dlgShiftPattern.verifyDialogValues(updateMinDuration40Days);
  });

  it(`Step 8 - ${StepDialog.updateDialogValues(DialogShiftPattern.title, verifyInvalidDurationNegative)}`, async () => {
    await dlgShiftPattern.updateDialogValues(verifyInvalidDurationNegative);
  });

  it(`Step 9 - ${StepDialog.verifyDialogValues(DialogShiftPattern.title, verifyValidDuration1Day)} ${StepDialog.verifyHasMaskError(DialogShiftPattern.title, {MinimumDuration: 'false'})}`, async () => {
    // Verify not possible key in negative sign, thus value becomes 1 and no mask error
    await dlgShiftPattern.verifyDialogValues(verifyValidDuration1Day);
    await dlgShiftPattern.verifyHasMaskError({MinimumDuration: 'false'});
  });

  it(`Step 10 - ${StepDialog.updateDialogValues(DialogShiftPattern.title, verifyValidDuration0Day)} ${StepDialog.verifyHasMaskError(DialogShiftPattern.title, {MinimumDuration: 'false'})}`, async () => {
    await dlgShiftPattern.updateDialogValues(verifyValidDuration0Day);
    await dlgShiftPattern.verifyHasMaskError({MinimumDuration: 'false'});
  });

  it(`Step 11 - ${StepDialog.updateDialogValues(DialogShiftPattern.title, verifyValidDuration40Days)} ${StepDialog.verifyHasMaskError(DialogShiftPattern.title, {MinimumDuration: 'false'})}`, async () => {
    await dlgShiftPattern.updateDialogValues(verifyValidDuration40Days);
    await dlgShiftPattern.verifyHasMaskError({MinimumDuration: 'false'});
  });

  it(`Step 12 - ${StepDialog.clickCancel()}`, async () => {
    await dlgShiftPattern.clickCancel();
  });
});
